package com.puzzledmonkey.db;

/**
 * Shuffle the sides of the cube. (6!=720)
 * Roll the initial cubes positions.
 * @author TomDalek
 */
public class DiceRoller {

    private int sides = 6;
    private int[] sidesVector = new int[sides];   //shuffled cube sides
    private int[][] cubesArray = new int[12][2];  //x,y positions of 12 cubes
    private int floorSize = 50; // in JME units (meters with the physics on)
    private int gridSize = 24; // grid is -gridSize..0..+gridSize
    private Grid grid;
    private Obstacle player1, player2, slot1, slot2;

    public DiceRoller() {
        shuffleCubeSides();
        initGrid();
        rollCubesPositions();
    }

    public int[] getSidesVector() {
        return sidesVector;
    }

    public String getSidesAsString(){
        String s = "shuffledSidesVector=";
        for (int i : sidesVector) {
            s = s + i;
        }
        return s;
    }

    public int[][] getcubesArray(){
        return cubesArray;
    }

    public int getFloorSize(){
        return floorSize;
    }

    @Override
    public String toString() {
        String s = getSidesAsString();
        s = s + "\n";

        for (int i = 0; i < 12; i++){
            s = s + (i+1) + ": [";
            for (int j = 0; j < 2; j++) {
                s = s + cubesArray[i][j];
                if (j==0) s = s + ",";
                }
            s = s + "]\n";
        }

        s = s + grid;

        return s;
    }

    /**
     * Populate the sidesVector with values 1-6,
     * value 0 in slotVector will mean an empty slot.
     */
    private void shuffleCubeSides() {
        // Populate the sidesVector
        for (int i = 0; i < sides; i++) {
            sidesVector[i] = i+1;
        }

        // shuffle
        for (int i = 0; i < sides; i++) {
            // (int) cast gives floor so not much probability of getting 5
            // but this is still good enough
            int r = (int) (Math.random() * (sides - 1));
            int swap = sidesVector[r];
            sidesVector[r] = sidesVector[i];
            sidesVector[i] = swap;
        }
    }

    private void initGrid(){
        grid = new Grid(gridSize);
        player1 = new Obstacle(1, 1, 0, 10, Obstacle.PLAYER);
        player2 = new Obstacle(1, 1, 0, -10, Obstacle.PLAYER);
        slot1 = new Obstacle(3, 2, -1, 2, Obstacle.SLOT);
        slot2 = new Obstacle(3, 2, -1, -3, Obstacle.SLOT);

        Obstacle other0 = new Obstacle(1, 1, 0, 0, Obstacle.OTHER);
        Obstacle other1 = new Obstacle(1, 1, -gridSize, gridSize, Obstacle.OTHER);
        Obstacle other2 = new Obstacle(1, 1, -gridSize, -gridSize, Obstacle.OTHER);
        Obstacle other3 = new Obstacle(1, 1, gridSize, -gridSize, Obstacle.OTHER);
        Obstacle other4 = new Obstacle(1, 1, gridSize, gridSize, Obstacle.OTHER);

        grid.putObstacle(player1);
        grid.putObstacle(player2);
        grid.putObstacle(slot1);
        grid.putObstacle(slot2);

        grid.putObstacle(other0);
//        grid.putObstacle(other1);
//        grid.putObstacle(other2);
//        grid.putObstacle(other3);
        grid.putObstacle(other4);
    }

    public Grid getGrid(){
        return grid;
    }

    public Obstacle getPlayer(int number){
        if (number == 1)
            return player1;
        else return player2;
    }

    public Obstacle getSlot(int number){
        if (number == 1)
            return slot1;
        else return slot2;
    }

    /**
     * Pick random xy, values for 12 cubes, with maximum of floorSize
     * and then shift it to the middle of coordinate system
     * with "-floorSize/2".
     */
    private void rollCubesPositions(){
        for (int i = 0; i < 12; i++){
            int x,y;
            do {
                x = Math.round((float)Math.random() * gridSize*2 - gridSize);
                y = Math.round((float)Math.random() * gridSize*2 - gridSize);
//                System.out.println("Rolling cube"+(i+1)+" position: ["+x+","+y+"]");
            } while (grid.getTile(x, y) !=  '.');

            cubesArray[i][0] = x;
            cubesArray[i][1] = y;
            
            Obstacle o = new Obstacle(1, 1, x, y, Obstacle.CUBE);
            grid.putObstacle(o);
        }
    }
}