package com.puzzledmonkey.db;

/**
 * The Grid is used mostly for quick cheking if a Grid's tile is already
 * occupied by an Obscale.<br><br>
 *
 * The coordinate system of the Grid is zero centric and Xright Yup oriented.
 * That is 0,0 is at the center.
 * (gridSize,gridSize) is in the top right corner.
 * (-gridSize,-gridSize) is in the bottom left corner. <br><br>
 * <code>
 * ........+gridSize..........<br>
 * -gridSize....0....+gridSize<br>
 * ........-gridSize..........<br>
 * </code>
 * @author TomDalek
 */
public class Grid {
    /**
     * Internaly we store the grid as an array:<br>
     * [raw (y+gridSize)][column (x+gridSize)],
     * The array is Yup oriented, so raw 0 is the bottom row,
     * last row is the top row.
     */
    private char[][] grid;
    private int gridSize;
    private int arraySize;

    Grid(int gridSize){
        this.gridSize = gridSize;
        this.arraySize = gridSize*2+1; // -gridSize..0..+gridSize
        this.grid = new char[arraySize][arraySize];
        for (int i=0; i < arraySize; i++)
            for (int j=0; j < arraySize; j++)
                this.grid[i][j] = Obstacle.EMPTY;
    }

    @Override
    public String toString() {
        String s = "The Grid:\n";
        
        // test header for gridSize=24
        s = s + "------------------------0++++++++++++++++++++++++\n";
        s = s + "222221111111111         0         111111111112222\n";
        s = s + "4321098765432109876543210123456789012345678901234\n";

        for (int i = arraySize-1; i >= 0; i--){ // because top row is last
            for (int j = 0; j < arraySize; j++) {
                s = s + grid[i][j];
                }
            s = s + "\n";
        }
        return s;
    }

    /**
     *
     * @param x -gridSize..0..+gridSize (column)
     * @param y -gridSize..0..+gridSize (row)
     * @param c ex Obstacle.PLAYER
     * @return TODO true if valid operation
     */
    public boolean  setTile(int x, int y, char c){
        grid[y+gridSize][x+gridSize] = c;
        return true; //TODO true if valid operation
    }

    /**
     *
     * @param x -gridSize..0..+gridSize (column)
     * @param y -gridSize..0..+gridSize (row)
     * @return TODO true if valid operation
     */
    public char getTile(int x, int y){
        return grid[y+gridSize][x+gridSize];
    }

    public boolean putObstacle(Obstacle o){
        for (int i = o.x; i < o.x+o.width; i++)
            for (int j = o.y; j < o.y+o.height; j++)
                setTile(i, j, o.c);
        return true; //TODO true if valid operation
    }

    public int getSize(){
        return gridSize;
    }
}
