package com.puzzledmonkey.db;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.URL;
import java.util.Enumeration;

/**
 * Finds internal (Ethernet) and external (Internet) IPs.
 * [ Taken from JPong :) ]
 * @author TomDalek
 */
public class IpRetriever {

    private String localIP;
    private String internetIP;

    public IpRetriever() {
        initLocalIP();
        initInternetIP();
    }

    @Override
    public String toString() {
        String s = "This computer's local IP:    " + localIP + "\n";
        s +=       "This computer's internet IP: " + internetIP + "\n";
        s +=       "We need to make sure the internet IP is added to the Allowed Hosts on our server.\n";
        return s;
    }
    public String getLocalIP(){
        return localIP;
    }
    public String getInternetIP(){
        return internetIP;
    }

    private void initLocalIP() {
        try {
            Boolean found = false;
            Enumeration<NetworkInterface> e1 = (Enumeration<NetworkInterface>) NetworkInterface.getNetworkInterfaces();
            while (e1.hasMoreElements()) {
                NetworkInterface ni = e1.nextElement();
                Enumeration<InetAddress> e2 = ni.getInetAddresses();
                while (e2.hasMoreElements()) {
                    InetAddress ia = e2.nextElement();
                    if (ia.isSiteLocalAddress()) {
                        this.localIP = ia.getHostAddress();
                        found = true;
                    }
                }
                if (!found) {
                    this.localIP = "Ethernet is down.";
                }
            }
        } catch (SocketException ex) {
            this.localIP = "Ethernet is down.";
        }
    }

    private void initInternetIP() {
        try {
            URL myIP = new URL("http://www.whatismyip.com/automation/n09230945.asp");
            BufferedReader in = new BufferedReader(new InputStreamReader(myIP.openStream()));
            String inputLine;
            while ((inputLine = in.readLine()) != null) {
                this.internetIP = inputLine;
            }
            in.close();
        } catch (Exception e) {
            this.internetIP = "Internet is down.";
        }
    }
}
