package com.puzzledmonkey.db;

import com.jme3.math.Vector2f;

/**
 * Obstacles are objects that are put on the Grid.
 * xy is the position of the bottom left corner of the obstacle!
 *
 * example:<br>
 * <code>
 * __ ------------------------0++++++++++++++++++++++++<br>
 * __ 222221111111111_________0_________111111111112222<br>
 * __ 4321098765432109876543210123456789012345678901234<br>
 * 02 .................................................<br>
 * 01 .................................................<br>
 * 00 ........................p........................<br>
 * -1 .................................................<br>
 * -2 .......................sss.......................<br>
 * -3 .......................sss.......................<br>
 * </code>
 * On this board:<br>
 * <code>
 * player1 = new Obstacle(1, 1,  0,  0, Obstacle.PLAYER);<br>
 *   slot1 = new Obstacle(3, 2, -1, -3, Obstacle.SLOT);<br>
 * </code>
 *
 * @author TomDalek
 */
public class Obstacle {
    public int width, height, x, y;
    public char c;

    public static final char
            EMPTY = '.',
            PLAYER = 'p',
            CUBE = 'c',
            SLOT = 's',
            OTHER = 'o';

    /**
     * @param width
     * @param height
     * @param x range: -gridSize..0..+gridSize-width
     * @param y range: -gridSize..0..+gridSize-height
     * @param c
     */
    Obstacle(int width, int height, int x, int y, char c){
        this.width = width;
        this.height = height;
        this.x = x;
        this.y = y;
        this.c = c;
    }

    public Vector2f getXY(){
        Vector2f xy = new Vector2f(x, y);
        return xy;
    }
}
