package com.puzzledmonkey.db;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Queue;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * MasterConnector uses this thread to periodicaly update (from the SQL server)
 * the logs added by the opponents.
 *
 * @author TomDalek
 */
class Updater extends Thread {

    Connection con = null;
    Queue<String> updateQueue;
    int playerID;

    Updater(Connection con, Queue<String> updateQueue, int playerID) {
        this.con = con;
        this.updateQueue = updateQueue;
        this.playerID = playerID;
    }

    @Override
    public void run() {
        long frequency = 30; // in Hz
        long period = 1000 / frequency;  // in milliseconds      
        long lastTimeMillis = System.currentTimeMillis();

        while (true) {
            if (System.currentTimeMillis() - lastTimeMillis > period) {
//                System.out.println("CurrentTimeMillis: " + System.currentTimeMillis());
//                System.out.println("lastTimeMillis: " + lastTimeMillis);
//                System.out.println("period: " + period);

//                updateQueue.add("Update: " + System.currentTimeMillis());

                gameUpdate();

                lastTimeMillis = System.currentTimeMillis();
            }

            // Sleep
            try {
                Thread.sleep(5);
            } catch (InterruptedException ex) {
                Logger.getLogger(Updater.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    void gameUpdate() {
        try {
            Statement st = con.createStatement();
            ResultSet rs = st.executeQuery("call pm_game_update()");
            while (rs.next()) {
                updateQueue.add(rs.getString(1));
                System.out.println(">>>> call pm_game_update() returned: " + rs.getString(1));
            }
        } catch (SQLException ex) {
            Logger.getLogger(MasterConnector.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
