package com.puzzledmonkey.jme;

import de.lessvoid.nifty.EndNotify;
import de.lessvoid.nifty.Nifty;
import de.lessvoid.nifty.controls.textfield.controller.TextFieldControl;
import de.lessvoid.nifty.effects.EffectEventId;
import de.lessvoid.nifty.elements.Element;
import de.lessvoid.nifty.input.NiftyInputEvent;
import de.lessvoid.nifty.input.mapping.DefaultInputMapping;
import de.lessvoid.nifty.screen.KeyInputHandler;
import de.lessvoid.nifty.screen.Screen;
import de.lessvoid.nifty.screen.ScreenController;

public class ChatController implements ScreenController, KeyInputHandler {

    private Nifty nifty;
    private Screen screen;
    private Element consoleElement;
    private ChatLogController chatLog;
    private boolean consoleVisible = false;
    private boolean allowConsoleToggle = true;
    private Element oldFocusElement;
    private Element consoleLayer;

    @Override
    public void bind(Nifty nifty, Screen screen) {
        this.nifty = nifty;
        this.screen = screen;
        screen.addKeyboardInputHandler(new DefaultInputMapping(), this);

        consoleElement = screen.findElementByName("text_input");
        chatLog = screen.findControl("chat_log", ChatLogController.class);
        consoleLayer = screen.findElementByName("input_layer");
    }

    @Override
    public void onStartScreen() {
        consoleElement.addInputHandler(new KeyInputHandler() {

            @Override
            public boolean keyEvent(NiftyInputEvent inputEvent) {
                if (inputEvent == null) {
                    return false;
                }
                switch (inputEvent) {
                    case SubmitText:
                        sendMessage();
                        return true;
                }
                return false;
            }
        });
        removeConsoleElementFromFocusHandler();
    }

    @Override
    public void onEndScreen() {
    }

    /**
     * Sends a message to the chat logs
     *
     * The chat mode is the default.
     * Everyting you write, goes to the chat as said by You.
     * If you prefix your message with "$ " you are sort of going into bash mode.
     *
     * ex writing message:
     * $ exit
     * will exit the game.
     */
    public void sendMessage() {
        String message = consoleElement.getControl(TextFieldControl.class).getText();

        PuzzledMonkey.masterMonkey.addCommand(message);

        consoleElement.getControl(TextFieldControl.class).setText("");
        consoleElement.setFocus();
    }

    public boolean keyEvent(final NiftyInputEvent inputEvent) {
        if (inputEvent == NiftyInputEvent.ConsoleToggle) {
            toggleConsole();
            return true;
        } else {
            return false;
        }
    }

    private void toggleConsole() {
        if (allowConsoleToggle) {
            allowConsoleToggle = false;
            if (consoleVisible) {
                closeConsole();
            } else {
                openConsole();
            }
        }
    }

    private void openConsole() {
        consoleLayer.showWithoutEffects();
        consoleLayer.startEffect(EffectEventId.onStartScreen, new EndNotify() {

            @Override
            public void perform() {
                oldFocusElement = screen.getFocusHandler().getKeyboardFocusElement();
                // add the consoleElement to the focushandler, when it's not yet added already
                addConsoleElementToFocusHandler();
                consoleElement.setFocus();

                consoleVisible = true;
                allowConsoleToggle = true;
            }
        });
    }

    private void closeConsole() {
        consoleLayer.startEffect(EffectEventId.onEndScreen, new EndNotify() {

            @Override
            public void perform() {
                consoleLayer.hideWithoutEffect();

                consoleVisible = false;
                allowConsoleToggle = true;

                if (oldFocusElement != null) {
                    oldFocusElement.setFocus();
                }

                removeConsoleElementFromFocusHandler();
            }
        });
    }

    private void addConsoleElementToFocusHandler() {
        if (screen.getFocusHandler().findElement(consoleElement.getId()) == null) {
            screen.getFocusHandler().addElement(consoleElement);
        }
    }

    private void removeConsoleElementFromFocusHandler() {
        screen.getFocusHandler().remove(consoleElement);
    }
}
