package com.puzzledmonkey.jme;

import com.jme3.bullet.collision.shapes.BoxCollisionShape;
import com.jme3.bullet.nodes.PhysicsNode;
import com.jme3.material.Material;
import com.jme3.math.FastMath;
import com.jme3.math.Quaternion;
import com.jme3.math.Vector3f;
import com.jme3.renderer.queue.RenderQueue.ShadowMode;
import com.jme3.scene.Geometry;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import com.jme3.scene.shape.Box;
import com.jme3.scene.VertexBuffer.Type;
import com.jme3.util.BufferUtils;
import java.util.Arrays;

/**
 *
 * @author maly
 */
public class Cube {

    public static class USER_DATA {

        final static String NUMBER = "Number",
                PLAYER = "Player",
                SLOT = "Slot",
                FACE = "Face";
    }
    private Box box;
    private float cubeSize = 0.5f;
    private PhysicsNode cubePhysNode;
    private int[] sidesVector;
//    private static final float[] GEOMETRY_TEXTURE_DATA = {
//        //(right, bottom),(left,bottom),(left,top),(right,top) CCW
//        1.000f, 0.5f, 0.666f, 0.5f, 0.666f, 1, 1.000f, 1, // back   (3) !!!
//        0.666f, 0.5f, 0.333f, 0.5f, 0.333f, 1, 0.666f, 1, // right  (2)
//        0.333f, 0.5f, 0.000f, 0.5f, 0.000f, 1, 0.333f, 1, // front  (1) !!!
//        0.333f, 0, 0, 0, 0, 0.5f, 0.333f, 0.5f, // left   (4)
//        0.666f, 0.5f, 0.666f, 0, 0.333f, 0, 0.333f, 0.5f, // top    (5)
//        0.666f, 0, 0.666f, 0.5f, 1.000f, 0.5f, 1.000f, 0 // bottom (6)
//    };
    private static final float[][] texureDataFront = {
        //(right, bottom),(left,bottom),(left,top),(right,top) CCW
        {0.333f, 0.5f, 0.000f, 0.5f, 0.000f, 1, 0.333f, 1}, // front  (1) !!!
        {0.666f, 0.5f, 0.333f, 0.5f, 0.333f, 1, 0.666f, 1}, // right  (2)
        {1.000f, 0.5f, 0.666f, 0.5f, 0.666f, 1, 1.000f, 1}, // back   (3) !!!
        {0.333f, 0, 0, 0, 0, 0.5f, 0.333f, 0.5f}, // left   (4)
        {0.666f, 0, 0.333f, 0, 0.333f, 0.5f, 0.666f, 0.5f}, // top    (5)
        {1.000f, 0, 0.666f, 0, 0.666f, 0.5f, 1.000f, 0.5f} // bottom (6)
    };
    // Last two go to the beggining (rotate pi/2 CW)
    private static final float[][] texureDataTop = {
        {0.333f, 1, 0.333f, 0.5f, 0.000f, 0.5f, 0.000f, 1}, // front  (1) !!!
        {0.666f, 1, 0.666f, 0.5f, 0.333f, 0.5f, 0.333f, 1}, // right  (2)
        {1.000f, 1, 1.000f, 0.5f, 0.666f, 0.5f, 0.666f, 1}, // back   (3) !!!
        {0.333f, 0.5f, 0.333f, 0, 0, 0, 0, 0.5f}, // left   (4)
        {0.666f, 0.5f, 0.666f, 0, 0.333f, 0, 0.333f, 0.5f}, // top    (5)
        {1.000f, 0.5f, 1.000f, 0, 0.666f, 0, 0.666f, 0.5f} // bottom (6)
    };
    // First two go to end (rotate pi/2 CCW)
    private static final float[][] texureDataBottom = {
        {0.000f, 0.5f, 0.000f, 1, 0.333f, 1, 0.333f, 0.5f}, // front  (1) !!!
        {0.333f, 0.5f, 0.333f, 1, 0.666f, 1, 0.666f, 0.5f}, // right  (2)
        {0.666f, 0.5f, 0.666f, 1, 1.000f, 1, 1.000f, 0.5f}, // back   (3) !!!
        {0, 0, 0, 0.5f, 0.333f, 0.5f, 0.333f, 0}, // left   (4)
        {0.333f, 0, 0.333f, 0.5f, 0.666f, 0.5f, 0.666f, 0}, // top    (5)
        {0.666f, 0, 0.666f, 0.5f, 1.000f, 0.5f, 1.000f, 0} // bottom (6)
    };

    public static enum Face {

        FRONT, RIGHT, BACK, LEFT, TOP, BOTTOM
    };
    public static float[][] faceRot = {
        {0, 0, 0},
        {0, -FastMath.HALF_PI, 0},
        {0, FastMath.PI, 0},
        {0, FastMath.HALF_PI, 0},
        {1.2f, 0, 0}, //{FastMath.HALF_PI, 0, 0},
        {-1.2f, 0, 0} //{-FastMath.HALF_PI, 0, 0}
    };
    public static final Quaternion YAW090 = new Quaternion().fromAngleAxis(FastMath.PI / 2, new Vector3f(0, 1, 0));

    // This requers explicit cast (float[])
//    public final static HashMap faceRot = new HashMap<Enum, float[]>();
//    static {
//        faceRot.put(Face.FRONT, new float[]{0, 0, 0});
//        faceRot.put(Face.RIGHT, new float[]{0, -FastMath.HALF_PI, 0});
//        faceRot.put(Face.BACK, new float[]{0, FastMath.PI, 0});
//        faceRot.put(Face.LEFT, new float[]{0, FastMath.HALF_PI, 0});
//        faceRot.put(Face.TOP, new float[]{-FastMath.HALF_PI, 0, 0});
//        faceRot.put(Face.BOTTOM, new float[]{FastMath.HALF_PI, 0, 0});
//    }
    Cube(PuzzledMonkey pm, Node parent, Vector3f position, int number, int[] sidesVector, Material mat) {
        this.sidesVector = sidesVector;
        // ground level is at y=4, y=0 is water level
        box = new Box(new Vector3f(0, 0, 0), cubeSize, cubeSize, cubeSize);
//        box.setBuffer(Type.TexCoord, 2, BufferUtils.createFloatBuffer(GEOMETRY_TEXTURE_DATA));
        box.setBuffer(Type.TexCoord, 2, BufferUtils.createFloatBuffer(generateTextureData()));

        Spatial cubeGeometry = new Geometry("CubeGeometry", box);
        cubeGeometry.setMaterial(mat);
        cubeGeometry.setShadowMode(ShadowMode.Cast);

        BoxCollisionShape boxShape =
                new BoxCollisionShape(new Vector3f(cubeSize, cubeSize, cubeSize));
        cubePhysNode = new PhysicsNode(cubeGeometry, boxShape, 0);
        cubePhysNode.setName("CubePhysNode");

        Node cube = new Node("Cube" + number);
        cube.setUserData(USER_DATA.NUMBER, number);
        cube.setUserData(USER_DATA.PLAYER, 0);
        cube.setUserData(USER_DATA.SLOT, 0);
        cube.setUserData(USER_DATA.FACE, 1);
        cube.setLocalTranslation(position);

        pm.getPhysicsSpace().add(cubePhysNode);
        cube.attachChild(cubePhysNode);
        parent.attachChild(cube);


//        cubePhysNode.rotate(-FastMath.HALF_PI, 0, 0); // For testing bottom face
    }

    static public int calcSide(Quaternion q) {
        float[] angles = q.toAngles(null);
        int side = 0;

        // sanitize the angles (make -0 into 0, and -pi into pi)
        for (int i = 0; i < angles.length; i++) {
            /*
             * Problem halfpi can should be 1.5707964
             * but sometimes it can be      1.5707965
             * What we want to do is round everything to the 1.0000 four digits
             * with someting like x = Math.round(x*10000.0f) / 10000.0f
             */

            float angle = Math.round(angles[i] * 10000.0f) / 10000.0f;

            if (angle == -0f) {
                angles[i] = 0f;
            }
            /*
             * Problem halfpi can be
             *    1.5707965
             * or 1.5707964
             * What we want to do is round everything to the 1.0000 four digits
             * with someting like x = Math.round(x*10000.0) / 10000.0
             */
            if ( (angle == Math.round(-FastMath.PI * 10000.0f) / 10000.0f) ||
                 (angle == Math.round( FastMath.PI * 10000.0f) / 10000.0f) ) {
                angles[i] = FastMath.PI;
            }
            if (angle == Math.round(-FastMath.HALF_PI * 10000.0f) / 10000.0f) {
                angles[i] = -FastMath.HALF_PI;
            }
            if (angle == Math.round(FastMath.HALF_PI * 10000.0f) / 10000.0f) {
                angles[i] = FastMath.HALF_PI;
            }
        }

        // deduct the front side from the rotation
        if (Arrays.equals(angles, faceRot[Face.FRONT.ordinal()])) {
            side = 1;
        } else if (Arrays.equals(angles, faceRot[Face.RIGHT.ordinal()])) {
            side = 2;
        } else if (Arrays.equals(angles, faceRot[Face.BACK.ordinal()])) {
            side = 3;
        } else if (Arrays.equals(angles, faceRot[Face.LEFT.ordinal()])) {
            side = 4;
        } else if (Arrays.equals(angles, faceRot[Face.TOP.ordinal()])) {
            side = 5;
        } else if (Arrays.equals(angles, faceRot[Face.BOTTOM.ordinal()])) {
            side = 6;
        }
        return side;
    }

    public String getFaceRotAsString(int face) {
        String s = "";

        for (float f : faceRot[face]) {
            s = s + f + ", ";
        }
        return s;
    }

    /**
     * Create shuffled UV mapping coordinates for the Cube
     * based on the DiceRoller's sidesVector.
     * @return new cordinate vector for the BufferUtils.createFloatBuffer() method
     */
    private float[] generateTextureData() {
        float[] f = new float[48]; // 8*6

        /*
         * BufferUtils.createFloatBuffer() method wants the faces to be in order:
         * Back, Right, Front, Left, Top, Bottom
         * but we use faces order (both in texureData and in sidesVector):
         * Front, Right, Back, Left, Top, Bottom
         *
         * So we flip the 0 and 2 faces
         */
        int[] textureSides = sidesVector.clone();
        textureSides[0] = sidesVector[2];
        textureSides[2] = sidesVector[0];

//        textureSides = new int[]{3,2,1,4,5,6};  // The propper order
//        textureSides = new int[]{3, 2, 1, 4, 5, 6};  // Test order

        System.out.println("TextureSides: " + textureSides[0] + textureSides[1] + textureSides[2] + textureSides[3] + textureSides[4] + textureSides[5]);

        int i = 0;
        for (int face = 0; face < 6; face++) {
//            System.out.print(face + ") ");
            for (int vert = 0; vert < 8; vert++) {
                if (face == 4) {
                    f[i] = texureDataTop[textureSides[face] - 1][vert];
                } else if (face == 5) {
                    f[i] = texureDataBottom[textureSides[face] - 1][vert];
                } else {
                    f[i] = texureDataFront[textureSides[face] - 1][vert];
                }

//                System.out.print(textureSides[face] - 1 + ", " + vert + "; " + f[i] + "||");
                i++;
            }
//            System.out.print("\n");
        }
        return f;
    }
}
