package com.puzzledmonkey.jme;

import com.jme3.animation.AnimChannel;
import com.jme3.animation.AnimControl;
import com.jme3.animation.AnimEventListener;
import com.jme3.animation.LoopMode;
import com.jme3.bullet.collision.shapes.BoxCollisionShape;
import com.jme3.bullet.collision.shapes.CapsuleCollisionShape;
import com.jme3.bullet.nodes.PhysicsCharacterNode;
import com.jme3.bullet.nodes.PhysicsNode;
import com.jme3.material.Material;
import com.jme3.math.ColorRGBA;
import com.jme3.math.FastMath;
import com.jme3.math.Quaternion;
import com.jme3.math.Vector3f;
import com.jme3.renderer.queue.RenderQueue.ShadowMode;
import com.jme3.scene.Geometry;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import com.jme3.scene.debug.Arrow;
import com.puzzledmonkey.db.DiceRoller;
import java.util.logging.Level;

/**
 *
 * @author TomDalek
 */
public class Player implements AnimEventListener {

    private PhysicsCharacterNode playerPhysNode;
    private Spatial model;
    private PhysicsNode slotsPhysNode;
    private Node inventory;
    private PuzzledMonkey pm;
    private AnimChannel channel;
    private AnimControl control;
    // walking and picking machine state
    private Vector3f target;
    private Geometry targetMark;
    private Node targetObject;
    private boolean onTheMove = false;       // true when walking
    private boolean seekingAnObject = false; // if walking after an object
    private boolean objectReached = false;   // when walking after an object finished
    private int[] slotsVector = new int[6];   //shuffled cube sides
    private int frontFace = 1;
    private Quaternion inventoryRotStart = new Quaternion();
    private Quaternion inventoryRotEnd = new Quaternion();
    private float inventoryRotProgress;
    private float inventoryRotMultip = 4; //tpf mltiplyer
    private float tpf;
    private boolean inventoryIsRotating;
    private DiceRoller dr;
    private int playerID;
    private float groundLevel;

    Player(PuzzledMonkey pm, Node parent, DiceRoller dr, int playerID, float groundLevel) {
        this.pm = pm;
        this.dr = dr;
        this.playerID = playerID;
        this.groundLevel = groundLevel;


        inventory = new Node("Inventory");

//        inventory.setQueueBucket(Bucket.Gui);
//        inventory.setLocalTranslation(0, 5, 0);
//        pm.getScene().attachChild(inventory);

        CapsuleCollisionShape capsule = new CapsuleCollisionShape(1f, 1f);
        playerPhysNode = new PhysicsCharacterNode(capsule, 0.01f);
        playerPhysNode.setName("PlayerPhysNode");
        model = (Node) pm.getAssetManager().loadModel("Models/Oto/Oto.mesh.xml");
        //        playerModel = (Node) assetManager.loadModel("Models/Sinbad/Sinbad.mesh.xml");
        //        playerModel = (Node) assetManager.loadModel("Models/Ninja/Ninja.mesh.xml");
        model.setLocalScale(0.31f); // Oto
        model.setShadowMode(ShadowMode.Cast);
        playerPhysNode.attachChild(model);
        playerPhysNode.setLocalTranslation(new Vector3f(
                dr.getPlayer(playerID).getXY().x,
                groundLevel,
                dr.getPlayer(playerID).getXY().y));
//        if (playerNumber == 2) {
//            playerPhysNode.setLocalRotation(new Quaternion().fromAngleNormalAxis(FastMath.PI, new Vector3f(0, 1, 0)));
//        }
        parent.attachChild(playerPhysNode);
        pm.getPhysicsSpace().add(playerPhysNode);

        /** Create a controller and channels. */
        control = model.getControl(AnimControl.class);
        control.addListener(this);
        channel = control.createChannel();
        channel.setAnim("stand");

        initTargetMark();
        initSlots(parent);

//        AxisRods ar = new AxisRods(pm.getAssetManager(), "Rods", 1f);
//        ar.setLocalTranslation(0, 0, 0);
//        inventory.attachChild(ar);
//        inventory.updateGeometricState();
    }

    private void initSlots(Node parent) {
        Node slots = (Node) pm.getAssetManager().loadModel("Scenes/Slots.j3o");
        slots.setName("Slots" + playerID);

        BoxCollisionShape boxShape =
                new BoxCollisionShape(new Vector3f(2f, 0.2f, 1f));
        slotsPhysNode = new PhysicsNode(slots, boxShape, 0);
        slotsPhysNode.setLocalTranslation(
                dr.getSlot(playerID).getXY().x + 1f,
                groundLevel,
                dr.getSlot(playerID).getXY().y + 0.5f);
        slotsPhysNode.setName("SlotsPhysNode" + playerID);
        if (playerID == 2) {
            slotsPhysNode.setLocalRotation(new Quaternion().fromAngleNormalAxis(FastMath.PI, new Vector3f(0, 1, 0)));
        }

        pm.getPhysicsSpace().add(slotsPhysNode);
        parent.attachChild(slotsPhysNode);

        /* Adding 6 Holders for all SlotsN>SlotN>HolderN
         * This is where the cubes will be instered.
         * We are useing this because cubes need to be extra rotated,
         * (facing up not front) when they are insterted into a slot.
         */
//        for (Spatial slot: slots.getChildren()){
        for (int i = 1; i <= 6; i++) {
            Node slot = (Node) slots.getChild("Slot" + i);

            Node holder = new Node();
            holder.setName("Holder" + i);
            holder.setLocalTranslation(new Vector3f(0, 0.5f + 0.1f, 0));
            Quaternion pitch090 = new Quaternion().fromAngleAxis(-FastMath.PI / 2, new Vector3f(1, 0, 0));
            holder.setLocalRotation(pitch090);
            slot.attachChild(holder);
        }
    }

    Node getSlotsPhysNode() {
        return slotsPhysNode;
    }

    PhysicsCharacterNode getPlayerPhysNode() {
        return playerPhysNode;
    }

    Node getInventory() {
        return inventory;
    }

    public String getInventoryRotAsString() {
        String s = "";
        float[] angles = inventory.getLocalRotation().toAngles(null);

        for (float f : angles) {
            s = s + f + ", ";
        }
        return s;
    }

    public int[] getSlotsVector() {
        return slotsVector;
    }

    public String getSlotsAsString() {
        String s = "slotsVector=";
        for (int i : slotsVector) {
            s = s + i;
        }
        return s;
    }

    int getFrontFace() {
        return frontFace;
    }

    Geometry getTargetMark() {
        return targetMark;
    }

    public void update() {
        if (onTheMove) {
            playerPhysNode.setWalkDirection(calcWalkDirection());

            // Rotete the model in the target direction
            Vector3f lookTarget = new Vector3f(
                    target.x,
                    playerPhysNode.getLocalTranslation().y,
                    target.z);
//            node.lookAt(lookTarget, Vector3f.UNIT_Y);
            playerPhysNode.getChild(0).lookAt(lookTarget, Vector3f.UNIT_Y);


            // Check for stoping conditions
            if (seekingAnObject) {
                if (calcFlatDistance() < 1.8) {
                    moveStop();
                    seekingAnObject = false;
                    objectReached = true;
                }
            } else if (calcFlatDistance() < 0.1) {
                moveStop();
            }


            if (objectReached) {
                objectReached = false;

                if (targetObject.getName().startsWith("Cube")) {
                    tryToPickUpCube();
                } else if (targetObject.getParent().getName().equals("Slots" + playerID)) {
                    putInSlot();
                }
            }
        }

        if (inventoryIsRotating) {
            if (inventoryRotProgress > 1) {
                inventoryIsRotating = false;
                inventory.setLocalRotation(inventoryRotEnd.clone());
                inventory.updateGeometricState();
            } else {
                Quaternion slerp = new Quaternion().slerp(inventoryRotStart, inventoryRotEnd, inventoryRotProgress);
                inventory.setLocalRotation(slerp);
                inventoryRotProgress += tpf * inventoryRotMultip;
                inventory.updateGeometricState();
            }
        }
    }

    public void move(Vector3f target) {
        pm.getChatLog().addLog("Moving to " + target.x + " " + target.z, ChatLogController.DEBUG);
        if (seekingAnObject) {
            pm.getChatLog().addLog("Abandoning seeking of " + targetObject.getName(), ChatLogController.DEBUG);
            seekingAnObject = false;
        }

        moveTo(target);
    }

    public void moveToGet(Node node) {
        seekingAnObject = true;
        targetObject = node;

        moveTo(node.getWorldTranslation());

        pm.getChatLog().addLog("Going to get " + node.getName() + " son of " + node.getParent().getName(), ChatLogController.DEBUG);
        pm.getLogger().log(Level.INFO,
                "|||||| Going to get: {0} son of {1}",
                new String[]{node.getName(), node.getParent().getName()});
    }

    private void moveTo(Vector3f target) {
        onTheMove = true;
        this.target = target;

        /* The actual moving:
         * node.setWalkDirection(calcWalkDirection());
         * is calculated in update() for every step,
         * because after hitting an obstacle, we nead to recalculate
         * the walking direction.
         */

        /** Play the "walk" animation! */
        channel.setAnim("Walk", 0.50f);
        channel.setLoopMode(LoopMode.Loop);
        channel.setSpeed(2f);
    }

    private void moveStop() {
        onTheMove = false;
        playerPhysNode.setWalkDirection(new Vector3f(0, 0, 0));
//        System.out.println("stoped");

        // Stop the animation
        channel.setAnim("stand", 0.50f);
        channel.setLoopMode(LoopMode.DontLoop);
        channel.setSpeed(1f);
    }

    private Vector3f calcWalkDirection() {
        Vector3f flatPosition = new Vector3f(
                playerPhysNode.getLocalTranslation().x, 0, playerPhysNode.getLocalTranslation().z);
        Vector3f flatTarget = new Vector3f(target.x, 0, target.z);

//        flatTarget.subtractLocal(flatPosition).normalizeLocal().divideLocal(8f);

        return flatTarget.subtract(flatPosition).normalize().divide(8f);
    }

    private float calcFlatDistance() {
        Vector3f flatPosition = new Vector3f(
                playerPhysNode.getLocalTranslation().x, 0, playerPhysNode.getLocalTranslation().z);
        Vector3f flatTarget = new Vector3f(target.x, 0, target.z);

        return flatPosition.distance(flatTarget);
    }

    private void initTargetMark() {
        Arrow arrowZ = new Arrow(Vector3f.UNIT_Z.mult(2f));
        arrowZ.setLineWidth(3);

        targetMark = new Geometry("Mark", arrowZ);
        Material mark_mat = new Material(pm.getAssetManager(), "Common/MatDefs/Misc/SolidColor.j3md");
        mark_mat.setColor("m_Color", ColorRGBA.Red);
        targetMark.setMaterial(mark_mat);

        //pm.getRootNode().attachChild(targetMark);
    }

    public void setDebug() {
        playerPhysNode.attachDebugShape(pm.getAssetManager());
    }

    /** Use this listener to trigger something after an animation is done. */
    public void onAnimCycleDone(AnimControl control, AnimChannel channel, String animName) {
//        throw new UnsupportedOperationException("Not supported yet.");
    }

    /** Use this listener to trigger something between two animations. */
    public void onAnimChange(AnimControl control, AnimChannel channel, String animName) {
//        throw new UnsupportedOperationException("Not supported yet.");
    }

    private void tryToPickUpCube() {
        Node cube = targetObject;
        Node cubes = targetObject.getParent();

        /* Check if the cube desn't belong to ther player */
        if (cube.getUserData(Cube.USER_DATA.PLAYER).equals(playerID)
                || cube.getUserData(Cube.USER_DATA.PLAYER).equals(0)) {

            /* Check if the inventory is empty */
            if (inventory.getChildren().isEmpty()) {
                pm.getChatLog().addLog("Picking up " + cube.getName(), ChatLogController.DEBUG);
                pm.getLogger().log(Level.INFO,
                        "|||||| Picking up: {0}", new String[]{cube.getName()});

                /* Check if the cube was in a slot */
                Integer slotNumber = (Integer) cube.getUserData(Cube.USER_DATA.SLOT);
                if (slotNumber != 0) {
                    pm.getChatLog().addLog("The cube was at a slot: " + slotNumber + ". Clearing the slot.", ChatLogController.DEBUG);
                    pm.getLogger().log(Level.INFO, "|||||| The cube was at a slot: {0}. Clearing the slot.", new String[]{slotNumber.toString()});

                    /* Clear the slot */
                    slotsVector[slotNumber - 1] = 0;

//                    /* Cubes in slots are extra rotate because slots are view from the top.
//                     * First we bring the proper rotation form the frontFace value.
//                     */
//                    Integer cubeFrontFace = (Integer) cube.getUserData(Cube.USER_DATA.FACE);
//                    cube.setLocalRotation(new Quaternion(Cube.faceRot[cubeFrontFace-1]));

                }

                /* Check if the slots arn't full.
                 * If the cube was in a slot at the begining the slots wan't be full
                 * at this pint, so it works for picking up a cube from full slots.
                 */
                if (!slotsAreFull()) {
                    pickUpCube(cubes, cube);
                } else {
                    pm.getChatLog().addLog("Your slots are full.", ChatLogController.SYSTEM);
                }
            } else {
                pm.getChatLog().addLog("You already have a cube in your inventory.", ChatLogController.SYSTEM);
            }
        } else {
            pm.getChatLog().addLog("This cube already belonges to your opponent. " + cube.getUserData(Cube.USER_DATA.PLAYER), ChatLogController.SYSTEM);
        }
    }

    private void pickUpCube(Node cubes, Node cube) {
        /* Cubes remember their rotation.
         * We want the invetorys rotation to inheret that rotation.
         */
        frontFace = (Integer) cube.getUserData(Cube.USER_DATA.FACE);
//        new Quaternion(Cube.faceRot[Cube.Face.BOTTOM.ordinal()]
        inventoryRotEnd.set(new Quaternion(Cube.faceRot[frontFace-1]));
        inventory.setLocalRotation(new Quaternion(Cube.faceRot[frontFace-1]));
        cube.setLocalRotation(Quaternion.IDENTITY);
        cube.setLocalTranslation(0, 0, 0);

//        cubes.detachChild(cube);
        inventory.attachChild(cube);
        pm.getPhysicsSpace().remove((PhysicsNode) cube.getChild("CubePhysNode"));
        inventory.updateGeometricState();
        cube.setUserData(Cube.USER_DATA.PLAYER, playerID);
    }

    private boolean slotsAreFull() {
        boolean slotIsFull = true;
        for (int i : slotsVector) {
            if (i == 0) {
                slotIsFull = false;
                break;
            }
        }
        return slotIsFull;
    }

    private void putInSlot() {
        if (!inventory.getChildren().isEmpty()) {
            Node cube = (Node) inventory.getChild(0);
            Node cubes = (Node) pm.getRootNode().getChild("Cubes");
            Node slot = targetObject;
            int slotNumber = Integer.parseInt(slot.getName().substring(slot.getName().length() - 1));
            Node holder = (Node) slot.getChild("Holder" + slotNumber);
            int faceAtTheSlot = slotsVector[slotNumber - 1];

            pm.getChatLog().addLog("Putting " + cube.getName() + " in " + slot.getName() + " son of " + slot.getParent().getName(), ChatLogController.DEBUG);

            if (faceAtTheSlot == 0) {
                /* Inheret the rotation from the inventory
                 */
                cube.setLocalRotation(inventory.getLocalRotation());

                /* When in inventory rot for 5 and 6 are not fully rotated
                 * so we need to set the rotation to a full rotation
                 */
                if (frontFace == 5) {
                    //this hack isn't nice but works :)
                    cube.setLocalRotation(new Quaternion().fromAngleAxis(FastMath.HALF_PI, new Vector3f(1, 0, 0)));
                }
                if (frontFace == 6) {
                    cube.setLocalRotation(new Quaternion().fromAngleAxis(-FastMath.HALF_PI, new Vector3f(1, 0, 0)));
                }

                inventory.detachAllChildren();
//                cubes.attachChild(cube);
                holder.attachChild(cube);
                pm.getPhysicsSpace().add((PhysicsNode) cube.getChild("CubePhysNode"));

                slotsVector[slotNumber - 1] = frontFace;
                cube.setUserData(Cube.USER_DATA.SLOT, slotNumber);
                cube.setUserData(Cube.USER_DATA.FACE, frontFace);
                inventory.updateGeometricState();
            } else {
                pm.getChatLog().addLog("This slot ins't empty.", ChatLogController.SYSTEM);
            }
        } else {
            pm.getChatLog().addLog("You don't have a cube in your inventory.", ChatLogController.SYSTEM);
        }
    }

    public void inventoryLeft(float tpf) {
        if (inventoryIsRotating) {
            return;
        }
        if (frontFace == 1 || frontFace == 2 || frontFace == 3 || frontFace == 4) {
            inventoryRotStart = new Quaternion(inventoryRotEnd);
            inventoryRotEnd = inventoryRotStart.mult(Cube.YAW090.inverse());
            inventoryRotProgress = 0;
            this.tpf = tpf;
            inventoryIsRotating = true;
            frontFace = Cube.calcSide(inventoryRotEnd);
        }
        inventory.updateGeometricState();
    }

    public void inventoryRight(float tpf) {
        if (inventoryIsRotating) {
            return;
        }
        if (frontFace == 1 || frontFace == 2 || frontFace == 3 || frontFace == 4) {
            inventoryRotStart = new Quaternion(inventoryRotEnd);
            inventoryRotEnd = inventoryRotStart.mult(Cube.YAW090);
            inventoryRotProgress = 0;
            this.tpf = tpf;
            inventoryIsRotating = true;
            frontFace = Cube.calcSide(inventoryRotEnd);
        }
        inventory.updateGeometricState();
    }

    public void inventoryUp(float tpf) {
//        System.out.println("invetoryUP");
        if (inventoryIsRotating) {
            return;
        }
        if (frontFace == 1 || frontFace == 2 || frontFace == 3 || frontFace == 4) {
            inventoryRotStart = new Quaternion(inventory.getLocalRotation());
            inventoryRotEnd = new Quaternion(Cube.faceRot[Cube.Face.BOTTOM.ordinal()]);
            inventoryRotProgress = 0;
            this.tpf = tpf;
            inventoryIsRotating = true;
            frontFace = Cube.calcSide(inventoryRotEnd);
        } else if (frontFace == 5) {
            inventoryRotStart = new Quaternion(inventory.getLocalRotation());
            inventoryRotEnd = new Quaternion(Cube.faceRot[Cube.Face.FRONT.ordinal()]);
            inventoryRotProgress = 0;
            this.tpf = tpf;
            inventoryIsRotating = true;
            frontFace = Cube.calcSide(inventoryRotEnd);
        }
        inventory.updateGeometricState();
    }

    public void inventoryDown(float tpf) {
//        System.out.println("invetoryDown");
        if (inventoryIsRotating) {
            return;
        }
        if (frontFace == 1 || frontFace == 2 || frontFace == 3 || frontFace == 4) {
            inventoryRotStart = new Quaternion(inventory.getLocalRotation());
            inventoryRotEnd = new Quaternion(Cube.faceRot[Cube.Face.TOP.ordinal()]);
            inventoryRotProgress = 0;
            this.tpf = tpf;
            inventoryIsRotating = true;
            frontFace = Cube.calcSide(inventoryRotEnd);
        } else if (frontFace == 6) {
            inventoryRotStart = new Quaternion(inventory.getLocalRotation());
            inventoryRotEnd = new Quaternion(Cube.faceRot[Cube.Face.FRONT.ordinal()]);
            inventoryRotProgress = 0;
            this.tpf = tpf;
            inventoryIsRotating = true;
            frontFace = Cube.calcSide(inventoryRotEnd);
        }
        inventory.updateGeometricState();
    }

    Boolean inventoryIsRotating() {
        return inventoryIsRotating;
    }

    float getInventoryRotProgress() {
        return inventoryRotProgress;
    }
}
