package com.puzzledmonkey.jme;

import com.jme3.app.SimpleApplication;
import com.jme3.asset.TextureKey;
import com.jme3.bullet.BulletAppState;
import com.jme3.bullet.PhysicsSpace;
import com.jme3.bullet.collision.shapes.BoxCollisionShape;
import com.jme3.bullet.nodes.PhysicsNode;
import com.jme3.collision.CollisionResult;
import com.jme3.collision.CollisionResults;
import com.jme3.effect.ParticleEmitter;
import com.jme3.effect.ParticleMesh.Type;
import com.jme3.font.BitmapText;
import com.jme3.input.KeyInput;
import com.jme3.input.MouseInput;
import com.jme3.input.controls.ActionListener;
import com.jme3.input.controls.KeyTrigger;
import com.jme3.input.controls.MouseButtonTrigger;
import com.jme3.light.DirectionalLight;
import com.jme3.material.Material;
import com.jme3.material.RenderState.BlendMode;
import com.jme3.math.ColorRGBA;
import com.jme3.math.FastMath;
import com.jme3.math.Quaternion;
import com.jme3.math.Ray;
import com.jme3.math.Vector2f;
import com.jme3.math.Vector3f;
import com.jme3.niftygui.NiftyJmeDisplay;
import com.jme3.renderer.Camera;
import com.jme3.renderer.ViewPort;
import com.jme3.renderer.queue.RenderQueue.ShadowMode;
import com.jme3.scene.Geometry;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import com.jme3.scene.debug.Arrow;
import com.jme3.scene.shape.Box;
import com.jme3.scene.shape.Quad;
import com.jme3.scene.shape.Sphere;
import com.jme3.shadow.BasicShadowRenderer;
import com.jme3.system.AppSettings;
import com.jme3.texture.Texture;
import com.jme3.texture.Texture.WrapMode;
import com.jme3.util.SkyFactory;
import com.jme3.water.SimpleWaterProcessor;
import com.puzzledmonkey.db.DiceRoller;
import com.puzzledmonkey.db.IpRetriever;
import com.puzzledmonkey.db.MasterConnector;
import de.lessvoid.nifty.Nifty;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.Queue;
import java.util.logging.Logger;

/** 
 * 1 cube = 1 puzzle.
 * Each of 6 cube faces = 1/6 of a puzzle.
 * 12 identical cubes. 
 * 2 players, 2 slots.
 */
public class PuzzledMonkey extends SimpleApplication {

    private Node sceneNode;  // things that are reflected in water
    private Node shootables; // things that are shootable
    private Node cubes;
    private BulletAppState bulletAppState;
    private PhysicsNode floorPhysNode;
    private PhysicsNode slotsPhysNode;
    private float groundLevel = 4.0f;
    private SimpleWaterProcessor waterProcessor;
    private Spatial waterPlane;
    private Vector3f lightPos = new Vector3f(-0.5f, -1f, -1f).normalizeLocal();
    private Geometry lightSphere;
    private BasicShadowRenderer bsr;
    private Material cubesMaterial;
    private Texture cubeTexture;
    private Texture testTexture;
    private boolean useTestTexture = false;
    private MyChaseCamera chaseCam;
    private Node chaseCamTarget;
    private Geometry mark;
    private Geometry markCube;
    private Node markSlot;
    private DiceRoller dr = new DiceRoller();
    private MasterConnector mc;
    private Player player;
    private Player opponent;
    private boolean debug = true;
    private Logger logger;
    private Node guiDebugNode;
    private BitmapText sidesText;
    private BitmapText answerText;
    private BitmapText slotsText;
    private BitmapText faceText;
    private BitmapText rotationText;
    private BitmapText rotationInProgressText;
    private Nifty nifty;
    private ChatLogController chatLog;
    //private ArrayList<String> chatList = new ArrayList<String>();
    private boolean gameSolved = false;
    private int[] answerVector = new int[6];
    private boolean useDiceRoller = true;
    public static PuzzledMonkey masterMonkey;
    private Queue<String> commandQueue = new LinkedList();
    private float tpf;
    private int playerID;
    private int opponentID;
    private int[] sidesVector = new int[6];   //shuffled cube sides
    private float time = 0;

    public static void main(String[] args) {
        AppSettings settings = new AppSettings(true);
        System.out.println(settings.getRenderer());

//        settings.setResolution(800, 600);
//        settings.setResolution(1024, 768);
        settings.setResolution(1280, 720);
//        settings.setResolution(1920, 1200);
//        settings.setFullscreen(true);
//        settings.setFrequency(50);

        settings.setSamples(4);
        settings.setFrameRate(80);
        PuzzledMonkey app = new PuzzledMonkey();
        app.setSettings(settings);
        app.setShowSettings(false);
        app.start();
    }

    @Override
    public void simpleInitApp() {
//        logger = Logger.getLogger(PuzzledMonkey.class.getName());
        logger = Logger.getLogger(SimpleApplication.class.getName());
//        logger.setLevel(Level.OFF);
//        Logger.getLogger(Logger.GLOBAL_LOGGER_NAME).setLevel(Level.OFF);
//        logger.setLevel(Level.FINE);

        masterMonkey = this;

        useDiceRoller = false;
        playerID = 2; // 1 = game creator, 2 = player
        mc = new MasterConnector(playerID);

//        if (playerID == 2) {
//            opponentID = 1;
//        } else {
//            opponentID = 2;
//        }

//        if (playerID == 1) {
//            mc.gameMake(24);
//        } else {
//            mc.gameJoin();
//        }

        playerID = mc.gameJoin();

        if (playerID == 2) {
            opponentID = 1;
        } else {
            opponentID = 2;
        }

        initScene();
        calcAnwserVector();
//        initWater();
        initMark();
        initKeys();
        initHUD();

//        commandQueue.add("$ get Cube1");
//        commandQueue.add("$ opget Cube2");
    }

    @Override
    public void simpleUpdate(float tpf) {

//        if (time > 0.5f) {
//          chatLog.addLog("time > 1.0f", ChatLogController.DEBUG);

//            time = 0;
//        } else {
//            time += tpf;
//        }

        commandQueue.addAll(mc.getUpdateQueue());

        this.tpf = tpf;
        colideMark(mark);
        player.update();
        opponent.update();

        slotsText.setText(player.getSlotsAsString());
        faceText.setText("frontFace=" + player.getFrontFace());
        rotationText.setText("Rotation=" + player.getInventoryRotAsString());
        rotationInProgressText.setText("Progress=" + player.getInventoryRotProgress() + "; " + player.inventoryIsRotating());

        gameSolve();
        commandSolve();
    }

    public Node getScene() {
        return sceneNode;
    }

    public ChatLogController getChatLog() {
        return chatLog;
    }

    public Logger getLogger() {
        return logger;
    }

    public String getSidesAsString() {
        String s = "shuffledSidesVector=";
        for (int i : sidesVector) {
            s = s + i;
        }
        return s;
    }

    private void initScene() {
        /** Set up Physics */
        bulletAppState = new BulletAppState();
        stateManager.attach(bulletAppState);

        sceneNode = new Node("Scene");
        initTextures();

        // Player
        player = new Player(this, sceneNode, dr, playerID, groundLevel);
        opponent = new Player(this, sceneNode, dr, opponentID, groundLevel);
//        player.setDebug();


        shootables = new Node("Shootables");
        initFloor(shootables);
        cubes = new Node("Cubes");
        initCubes(cubes);
        shootables.attachChild(player.getSlotsPhysNode());
        shootables.attachChild(cubes);
        sceneNode.attachChild(shootables);


        // Camera
        flyCam.setEnabled(false);
        chaseCamTarget = new Node("CamTarget");
        chaseCamTarget.setLocalTranslation(0, 1.4f, 0);
        player.getPlayerPhysNode().attachChild(chaseCamTarget);
        chaseCam = new MyChaseCamera(cam, chaseCamTarget, inputManager);
        chaseCam.setSmoothMotion(true);
        chaseCam.setDefaultHorizontalRotation(FastMath.HALF_PI);
        chaseCam.setMaxVerticalRotation(FastMath.HALF_PI);
        chaseCam.setMinVerticalRotation(-0.01f);
//        chaseCam.setMinVerticalRotation(-20.0f); // testing
        chaseCam.setRotationSensitivity(20.0f);
        chaseCam.setMaxDistance(100f);


        // Light
        DirectionalLight sun = new DirectionalLight();
        sun.setDirection(lightPos);
        sun.setColor(ColorRGBA.White);
        sceneNode.addLight(sun);


        //add lightPos Geometry
        Sphere lite = new Sphere(8, 8, 0.1f);
        lightSphere = new Geometry("lightsphere", lite);
        Material sphereMat = new Material(assetManager, "Common/MatDefs/Misc/SimpleTextured.j3md");
        sphereMat.setTexture("m_ColorMap", assetManager.loadTexture("Interface/Logo/Monkey.jpg"));
        lightSphere.setMaterial(sphereMat);
        lightSphere.setLocalTranslation(lightPos.add(0, 5, 0));
        sceneNode.attachChild(lightSphere);

        // Shadow
//        sceneNode.setShadowMode(ShadowMode.Off);
////        bsr = new BasicShadowRenderer(assetManager, 256);
//        bsr = new BasicShadowRenderer(assetManager, 1024 * 2);
//        bsr.setDirection(lightPos);
//        viewPort.addProcessor(bsr);


        // Sky
        sceneNode.attachChild(SkyFactory.createSky(
                assetManager, "Textures/Sky/Bright/BrightSky.dds", false));


        //Bloom
//        FilterPostProcessor fpp = new FilterPostProcessor(assetManager);
//        BloomFilter bloom = new BloomFilter();
////        bloom.setBlurScale(1.0f);
//        bloom.setExposurePower(30.0f);
//        fpp.addFilter(bloom);
//        viewPort.addProcessor(fpp);

        // AxisRods
        AxisRods ar = new AxisRods(assetManager, "Rods", 1f);
        ar.setLocalTranslation(0f, groundLevel, 0f);
        rootNode.attachChild(ar);

        rootNode.attachChild(sceneNode);
    }

    private void initTextures() {
        TextureKey cubeTextureKey = new TextureKey("Textures/Tunisia.jpg");
        cubeTextureKey.setGenerateMips(true);
        cubeTexture = assetManager.loadTexture(cubeTextureKey);

        TextureKey testTextureKey = new TextureKey("Textures/testmap.jpg");
        testTextureKey.setGenerateMips(true);
        testTexture = assetManager.loadTexture(testTextureKey);
    }

    /**
     * CubePhysNode will have 2 UserData:
     * Integer "Number" - this cubes number.
     * Integer "SlotNumber" - slot's number if in a slot, null if on the floor.
     * 
     * @param parent A Node to which the cubes will be attached to.
     */
    private void initCubes(Node parent) {
        int[][] cubesArray = new int[12][2];  //x,y positions of 12 cubes

        if (useDiceRoller) {
            cubesArray = dr.getcubesArray();
            sidesVector = dr.getSidesVector();
        } else {
            cubesArray = mc.gameGet();
            sidesVector = mc.gameGetSides();
        }

        cubesMaterial = new Material(assetManager, "Common/MatDefs/Misc/SimpleTextured.j3md");
        cubesMaterial.setTexture("m_ColorMap", cubeTexture);

        for (int i = 0; i < 12; i++) {
            Vector3f position = new Vector3f(cubesArray[i][0], groundLevel + 0.5f, cubesArray[i][1]);
            new Cube(this, parent, position, i + 1, sidesVector, cubesMaterial);
        }
    }

    /** A floor to show that the "shot" can go through several objects. */
    protected void initFloor(Node parent) {
        /* Material - alpha blend */
//        Material floor_mat = new Material(assetManager, "Common/MatDefs/Misc/SolidColor.j3md");
//        floor_mat.setColor("m_Color", new ColorRGBA(1,0.5f,0.5f,0.5f));
//        floor_mat.getAdditionalRenderState().setBlendMode(BlendMode.Alpha);

        /* Material - SimpleTextured */
        Material floor_mat = new Material(assetManager, "Common/MatDefs/Misc/SimpleTextured.j3md");
        TextureKey floorKey = new TextureKey("Textures/Terrain/Pond/Pond.png");
        floorKey.setGenerateMips(true);
        Texture floorTex = assetManager.loadTexture(floorKey);
        floorTex.setWrap(WrapMode.Repeat);
        floor_mat.setTexture("m_ColorMap", floorTex);

        /* Material - Lighting */
//        Material floor_mat = new Material(assetManager, "Common/MatDefs/Light/Lighting.j3md");
//        floor_mat.setTexture("m_DiffuseMap",
//                assetManager.loadTexture("Textures/Terrain/Pond/Pond.png"));
//        floor_mat.setTexture("m_NormalMap",
//                assetManager.loadTexture("Textures/Terrain/Pond/Pond_normal.png"));
//        floor_mat.setFloat("m_Shininess", 5f); // [0,128]


        /* Geometry - Box
         * Ground level is at y=4, y=0 is water level
         */
        int boxSize = dr.getFloorSize() / 2;
        Box box = new Box(new Vector3f(0, 3f, 0), boxSize, 1f, boxSize);
        box.scaleTextureCoordinates(new Vector2f(5, 5));
        Spatial floor = new Geometry("Floor", box);
        floor.setMaterial(floor_mat);
        floor.setShadowMode(ShadowMode.Receive);

        /* Geometry - Quad */
//        Quad quad = new Quad(boxSize*2, boxSize*2);
//        Geometry floor = new Geometry("Floor", quad);
//        floor.rotate(new Quaternion().fromAngleAxis(-FastMath.PI/2, new Vector3f(1,0,0)));
//        floor.setLocalTranslation(new Vector3f(-boxSize,4f,boxSize));
//        FloatBuffer tbuf = quad.getFloatBuffer(Type.TexCoord);
//        float repeatY = 20;
//        float repeatX = 20;
//
//        tbuf.clear();
//        tbuf.put(0).put(repeatY);
//        tbuf.put(0).put(0);
//        tbuf.put(repeatX).put(0);
//        tbuf.put(repeatX).put(repeatY);



        /* Physics */
        BoxCollisionShape boxShape =
                new BoxCollisionShape(new Vector3f(boxSize, 4f, boxSize));
        floorPhysNode = new PhysicsNode(floor, boxShape, 0);
        floorPhysNode.setName("FloorNode");

        getPhysicsSpace().add(floorPhysNode);
        parent.attachChild(floorPhysNode);
    }

    private void initWater() {
        //create processor
        waterProcessor = new SimpleWaterProcessor(assetManager);
        waterProcessor.setReflectionScene(sceneNode);
//        waterProcessor.setDebug(true);
        viewPort.addProcessor(waterProcessor);

//        waterProcessor.setLightPosition(lightPos);

        // we set wave properties
//        waterProcessor.setWaterDepth(4);         // transparency of water
        waterProcessor.setWaterDepth(0);         // transparency of water
        waterProcessor.setDistortionScale(0.1f); // strength of waves
        waterProcessor.setWaveSpeed(0.02f);       // speed of waves


        //create water quad
        waterPlane = waterProcessor.createWaterGeometry(100, 100);
        waterPlane.setMaterial(waterProcessor.getMaterial());
        waterPlane.setLocalScale(20);
        waterPlane.setLocalTranslation(-1000, 0, 1000);
//        waterPlane.setLocalScale(10);
//        waterPlane.setLocalTranslation(-500, 0, 500);

//        Material m_wire = new Material(assetManager, "Common/MatDefs/Misc/WireColor.j3md");
//        m_wire.setColor("m_Color", ColorRGBA.Green);
//        waterPlane.setMaterial(m_wire);

        rootNode.attachChild(waterPlane);


    }

    protected void initMark() {
        Arrow arrowZ = new Arrow(Vector3f.UNIT_Z.mult(2f));
        arrowZ.setLineWidth(3);

        mark = new Geometry("Mark", arrowZ);
        Material mark_mat = new Material(assetManager, "Common/MatDefs/Misc/SolidColor.j3md");
        mark_mat.setColor("m_Color", ColorRGBA.Blue);
        mark.setMaterial(mark_mat);

        rootNode.attachChild(mark);


        Sphere lite = new Sphere(8, 8, 0.4f);
        markCube = new Geometry("MarkCube", lite);
        Material sphereMat = new Material(assetManager, "Common/MatDefs/Misc/SimpleTextured.j3md");
        sphereMat.setTexture("m_ColorMap", assetManager.loadTexture("Interface/Logo/Monkey.jpg"));
        markCube.setMaterial(sphereMat);
        markCube.setLocalTranslation(lightPos.add(0, 5, 0));
        rootNode.attachChild(markCube);

        markSlot = new Node();
        Sphere markSlotSphere = new Sphere(8, 8, 0.04f);
        Geometry markSlotBall = new Geometry("MarkSlotBall", markSlotSphere);
        sphereMat.setTexture("m_ColorMap", assetManager.loadTexture("Interface/Logo/Monkey.jpg"));
        markSlotBall.setMaterial(sphereMat);
        markSlotBall.setLocalTranslation(lightPos.add(0, 5, 0));
        markSlot.attachChild(markSlotBall);

        //-----------
//        ParticleEmitter emit = new ParticleEmitter("Emitter", Type.Triangle, 200);
//        emit.setGravity(0);
//        emit.setVariation(1);
//        emit.setLowLife(1);
//        emit.setHighLife(1);
//        emit.setStartVel(new Vector3f(0, .5f, 0));
//        emit.setImagesX(15);
//        Material mat = new Material(assetManager, "Common/MatDefs/Misc/Particle.j3md");
//        mat.setTexture("Texture", assetManager.loadTexture("Effects/Smoke/Smoke.png"));
//        emit.setMaterial(mat);
//
//        markSlot.attachChild(emit);
        //-----------


//        rootNode.attachChild(markSlot);
    }

    /** Declaring the "Shoot" action and mapping to its triggers. */
    private void initKeys() {
        inputManager.deleteMapping("SIMPLEAPP_Exit");
        inputManager.addMapping("Exit", new KeyTrigger(KeyInput.KEY_ESCAPE));
        inputManager.deleteMapping("SIMPLEAPP_CameraPos");
        inputManager.deleteMapping("SIMPLEAPP_Memory");


        inputManager.addMapping("Shoot", new MouseButtonTrigger(MouseInput.BUTTON_LEFT));
        inputManager.addMapping("View", new MouseButtonTrigger(MouseInput.BUTTON_RIGHT));
        inputManager.addMapping("Left", new KeyTrigger(KeyInput.KEY_LEFT));
        inputManager.addMapping("Right", new KeyTrigger(KeyInput.KEY_RIGHT));
        inputManager.addMapping("Up", new KeyTrigger(KeyInput.KEY_UP));
        inputManager.addMapping("Down", new KeyTrigger(KeyInput.KEY_DOWN));
        inputManager.addMapping("Test", new KeyTrigger(KeyInput.KEY_F2));
        inputManager.addMapping("DebugGUI", new KeyTrigger(KeyInput.KEY_F3));

        inputManager.addListener(actionListener, new String[]{"Shoot", "Right", "Left", "Up", "Down", "Test", "View", "DebugGUI", "Exit"});
    }
    /** Defining the "Shoot" action: Determine what was hit and how to respond. */
    private ActionListener actionListener = new ActionListener() {

        public void onAction(String name, boolean keyPressed, float tpf) {
            if (name.equals("Shoot") && !keyPressed) {
                CollisionResult target = colideMark(player.getTargetMark());

//                System.out.print("Shot target: " + target.getGeometry().getParent().getParent().getParent().getParent().getParent().getParent());
//                System.out.print(" > " + target.getGeometry().getParent().getParent().getParent().getParent().getParent());
//                System.out.print(" > " + target.getGeometry().getParent().getParent().getParent().getParent());
//                System.out.print(" > " + target.getGeometry().getParent().getParent().getParent());
//                System.out.print(" > " + target.getGeometry().getParent().getParent());
//                System.out.print(" > " + target.getGeometry().getParent());
//                System.out.println(" > " + target.getGeometry());


                if (target != null) {
                    //Scene (Node) > Shootables (Node) > FloorNode (PhysicsNode) > Floor (Geometry)
                    if (target.getGeometry().getParent() == floorPhysNode) {
//                        player.move(target.getContactPoint());
                        commandQueue.add("$ move " + target.getContactPoint().x + " " + target.getContactPoint().z);

                        //Shot target: Root Node (Node) > Scene (Node) > Shootables (Node) > Cubes (Node) > Cube10 (Node) > CubePhysNode10 (PhysicsNode) > CubeGeometry10 (Geometry)
//                    } else if (target.getGeometry().getParent().getParent().getParent() == cubes) {
                    } else if (target.getGeometry().getName().equals("CubeGeometry")) {
                        Node cube = target.getGeometry().getParent().getParent();
//                        player.moveToGet(cube);
                        addCommand("$ get " + cube.getName());

                        //Shot target: Shootables (Node) > SlotsPhysNode1 (PhysicsNode) > Slots1 (Node) > Slot4 (Node) > Slot4-entity (Node) > Slot4-ogremesh (Node) > Slot4-geom-1 (Geometry)
                    } else if (target.getGeometry().getParent().getParent().getParent().getParent().getName().equals("Slots" + playerID)) {
                        Node slot = target.getGeometry().getParent().getParent().getParent();
//                        player.moveToGet(slot);
                        addCommand("$ get " + slot.getName());
                    }
                }

            }

            if (name.equals("Left") && !keyPressed) {
//                player.inventoryLeft(tpf);
                addCommand("$ rot left");
            }
            if (name.equals("Right") && !keyPressed) {
//                player.inventoryRight(tpf);
                addCommand("$ rot right");
            }
            if (name.equals("Up") && !keyPressed) {
//                player.inventoryUp(tpf);
                addCommand("$ rot up");
            }
            if (name.equals("Down") && !keyPressed) {
//                player.inventoryDown(tpf);
                addCommand("$ rot down");
            }
            if (name.equals("Test") && !keyPressed) {
                if (!useTestTexture) {
                    cubesMaterial.setTexture("m_ColorMap", testTexture);
                } else {
                    cubesMaterial.setTexture("m_ColorMap", cubeTexture);
                }

                useTestTexture = !useTestTexture;
            }
            if (name.equals("View") && !keyPressed) {
                viewSwich();
            }
            if (name.equals("DebugGUI") && !keyPressed) {
                if (guiDebugNode.getParent() == null) {
                    guiNode.attachChild(guiDebugNode);
                } else {
                    guiNode.detachChild(guiDebugNode);
                }

            }
            if (name.equals("Exit") && !keyPressed) {
                stopPuzzledMonkey();
            }

        }

        private void viewSwich() {
            Node camTargetParent = chaseCamTarget.getParent();
            if (camTargetParent.getName().equals("PlayerPhysNode")) {
                player.getSlotsPhysNode().attachChild(chaseCamTarget);
            } else {
                player.getPlayerPhysNode().attachChild(chaseCamTarget);
            }
//            System.out.println(""+chaseCamTarget.getParent());
        }
    };

    public PhysicsSpace getPhysicsSpace() {
        return bulletAppState.getPhysicsSpace();
    }

    private CollisionResult colideMark(Geometry mark) {
        rootNode.updateGeometricState();
        CollisionResult closest = null;

        Vector3f origin = cam.getWorldCoordinates(inputManager.getCursorPosition(), 0.0f);
        Vector3f direction = cam.getWorldCoordinates(inputManager.getCursorPosition(), 0.3f);
        direction.subtractLocal(origin).normalizeLocal();

        Ray ray = new Ray(origin, direction);
        CollisionResults results = new CollisionResults();
        shootables.collideWith(ray, results);

        if (results.size() > 0) {
            closest = results.getClosestCollision();
            mark.setLocalTranslation(closest.getContactPoint());

            Quaternion q = new Quaternion();
            q.lookAt(closest.getContactNormal(), Vector3f.UNIT_Y);
            mark.setLocalRotation(q);

            rootNode.attachChild(mark);



            if (closest.getGeometry().getParent().getParent().getParent() == cubes) {
                Node cube = closest.getGeometry().getParent().getParent();

                markCube.setLocalTranslation(new Vector3f(
                        cube.getLocalTranslation().x,
                        groundLevel + 1.5f,
                        cube.getLocalTranslation().z));
                rootNode.attachChild(markCube);
            } else {
                rootNode.detachChild(markCube);
            }

            if (closest.getGeometry().getParent().getParent().getParent().getParent().getName().equals("Slots" + playerID)) {
                Node slot = closest.getGeometry().getParent().getParent().getParent();

                markCube.setLocalTranslation(new Vector3f(
                        slot.getWorldTranslation().x,
                        groundLevel + 0.5f,
                        slot.getWorldTranslation().z));
                rootNode.attachChild(markCube);
            } else {
                rootNode.detachChild(markSlot);
            }

        } else {
            rootNode.detachChild(mark);
            rootNode.detachChild(markCube);
            rootNode.detachChild(markSlot);
        }
        return closest;
    }

    private void initHUD() {
        int w = settings.getWidth();
        int h = settings.getHeight();

//        guiNode.detachChildNamed("Statistics View");
        guiNode.detachAllChildren();

        guiDebugNode = new Node("GuiDebugNode");

        /* Backgrounds */
        Material hud_mat = new Material(assetManager, "Common/MatDefs/Misc/SolidColor.j3md");
        hud_mat.setColor("m_Color", new ColorRGBA(0.8f, 0.6f, 0.4f, 0.5f));
        hud_mat.getAdditionalRenderState().setBlendMode(BlendMode.Alpha);

        Quad topQuad = new Quad(240, 196);
        Geometry topHUD = new Geometry("TopHUD", topQuad);
        topHUD.setMaterial(hud_mat);
        topHUD.setLocalTranslation(6, h - 6 - 196, -1);
        guiDebugNode.attachChild(topHUD);

//        Quad bottomQuad = new Quad(180, 166);
//        Geometry bottomHUD = new Geometry("BottomHUD", bottomQuad);
//        bottomHUD.setMaterial(hud_mat);
//        bottomHUD.setLocalTranslation(0, 0, -1);
//        guiNode.attachChild(bottomHUD);

        /* Texts */
//        guiFont = assetManager.loadFont("Interface/Fonts/Default.fnt");

        sidesText = new BitmapText(guiFont, false);
        sidesText.setSize(guiFont.getCharSet().getRenderedSize());
        sidesText.setColor(ColorRGBA.White);
        sidesText.setText(getSidesAsString());
        sidesText.setLocalTranslation(10, h - 10, 0);
        sidesText.setName("Sides");
        guiDebugNode.attachChild(sidesText);

        answerText = new BitmapText(guiFont, false);
        answerText.setSize(guiFont.getCharSet().getRenderedSize());
        answerText.setColor(ColorRGBA.White);
        answerText.setText(getAnwserAsString());
        answerText.setLocalTranslation(10, h - 10 - answerText.getLineHeight(), 0);
        answerText.setName("Sides");
        guiDebugNode.attachChild(answerText);

        slotsText = new BitmapText(guiFont, false);
        slotsText.setSize(guiFont.getCharSet().getRenderedSize());
        slotsText.setColor(ColorRGBA.White);
        slotsText.setText(player.getSlotsAsString());
        slotsText.setLocalTranslation(10, h - 10 - 2 * slotsText.getLineHeight(), 0);
        slotsText.setName("Slots");
        guiDebugNode.attachChild(slotsText);

        faceText = new BitmapText(guiFont, false);
        faceText.setSize(guiFont.getCharSet().getRenderedSize());
        faceText.setColor(ColorRGBA.White);
        faceText.setText("frontFace=" + player.getFrontFace());
        faceText.setLocalTranslation(10, h - 10 - 3 * faceText.getLineHeight(), 0);
        faceText.setName("Face");
        guiDebugNode.attachChild(faceText);

        rotationText = new BitmapText(guiFont, false);
        rotationText.setSize(guiFont.getCharSet().getRenderedSize());
        rotationText.setColor(ColorRGBA.White);
        rotationText.setText("Rotation=" + player.getInventory().getLocalRotation());
        rotationText.setLocalTranslation(10, h - 10 - 4 * rotationText.getLineHeight(), 0);
        rotationText.setName("Rotation");
        guiDebugNode.attachChild(rotationText);

        rotationInProgressText = new BitmapText(guiFont, false);
        rotationInProgressText.setSize(guiFont.getCharSet().getRenderedSize());
        rotationInProgressText.setColor(ColorRGBA.White);
        rotationInProgressText.setText("Inventory is rotating = ");
        rotationInProgressText.setLocalTranslation(10, h - 10 - 5 * rotationInProgressText.getLineHeight(), 0);
        rotationInProgressText.setName("Rotation");
        guiDebugNode.attachChild(rotationInProgressText);

        /* IP's */
        IpRetriever ip = new IpRetriever();
        BitmapText localIPtext = new BitmapText(guiFont, false);
        localIPtext.setSize(guiFont.getCharSet().getRenderedSize());
        localIPtext.setColor(ColorRGBA.White);
        localIPtext.setText("LocalIP=" + ip.getLocalIP());
        localIPtext.setLocalTranslation(10, h - 10 - 6 * localIPtext.getLineHeight(), 0);
        localIPtext.setName("Rotation");
        guiDebugNode.attachChild(localIPtext);

        BitmapText internetIPtext = new BitmapText(guiFont, false);
        internetIPtext.setSize(guiFont.getCharSet().getRenderedSize());
        internetIPtext.setColor(ColorRGBA.White);
        internetIPtext.setText("InternetIP=" + ip.getInternetIP());
        internetIPtext.setLocalTranslation(10, h - 10 - 7 * internetIPtext.getLineHeight(), 0);
        internetIPtext.setName("Rotation");
        guiDebugNode.attachChild(internetIPtext);

        fpsText = new BitmapText(guiFont, false);
        fpsText.setLocalTranslation(0, fpsText.getLineHeight(), 0);
        fpsText.setLocalTranslation(10, h - 10 - 8 * internetIPtext.getLineHeight(), 0);
        guiDebugNode.attachChild(fpsText);

        guiNode.attachChild(guiDebugNode);

        /* Inventory */
        DirectionalLight inventoryLamp = new DirectionalLight();
        inventoryLamp.setDirection(Vector3f.UNIT_XYZ.negate());
        player.getInventory().addLight(inventoryLamp);

        Camera inventoryCam = cam.clone();
        float size = 0.4f;
        inventoryCam.setViewPort(1.05f - size * (1280 / 720), 1.05f, 0.0f, 0.0f + size);
        inventoryCam.setLocation(new Vector3f(0, 0, 1.92f));
        inventoryCam.lookAt(new Vector3f(0, 0, 0), Vector3f.UNIT_Y);
        renderManager.setCamera(inventoryCam, false);

        ViewPort inventoryView = renderManager.createMainView("InventoryView", inventoryCam);
//        inventoryView.setClearEnabled(true);
        inventoryView.attachScene(player.getInventory());


        /* Opponent Invetory */
        DirectionalLight opponentInventoryLamp = new DirectionalLight();
        opponentInventoryLamp.setDirection(Vector3f.UNIT_XYZ.negate());
        player.getInventory().addLight(opponentInventoryLamp);

        Camera opponentInventoryCam = cam.clone();
//        float size = 0.4f;
        opponentInventoryCam.setViewPort(-0.05f, -0.05f + size * (1280 / 720), 0.0f, 0.0f + size);
        opponentInventoryCam.setLocation(new Vector3f(0, 0, 1.92f));
        opponentInventoryCam.lookAt(new Vector3f(0, 0, 0), Vector3f.UNIT_Y);
        renderManager.setCamera(opponentInventoryCam, false);

        ViewPort opponentInventoryView = renderManager.createMainView("OpponentInventoryView", opponentInventoryCam);
//        opponentInventoryView.setClearEnabled(true);
        opponentInventoryView.attachScene(opponent.getInventory());



        /* Chat */
        NiftyJmeDisplay niftyDisplay = new NiftyJmeDisplay(
                assetManager, inputManager, audioRenderer, guiViewPort);
        nifty = niftyDisplay.getNifty();
        nifty.fromXml("Interface/chat.xml", "start");
        chatLog = nifty.getScreen("start").findControl("chat_log", ChatLogController.class);

        // attach the nifty display to the gui view port as a processor
        guiViewPort.addProcessor(niftyDisplay);

//        for (int i=0; i < 40; i++){
//            chat.addLog("Chat log test " + i);
//        }
        chatLog.addLog("Press F1 to toggle chat input.", ChatLogController.HELP);
        chatLog.addLog("Press F2 to toggle testmap texture.", ChatLogController.HELP);
        chatLog.addLog("Anonymous text.");
        chatLog.addLog("I'am whaching over.", ChatLogController.DEBUG);
        chatLog.addLog("I will advise the player about his actions.", ChatLogController.SYSTEM);
        chatLog.addLog("This mokney is like the coolest.", ChatLogController.YOU);
        chatLog.addLog("Oh yeah! :)", ChatLogController.OPPONENT);
    }

    private void gameSolve() {
        if (!gameSolved) {
            if (Arrays.equals(player.getSlotsVector(), answerVector)) {
                answerText.setColor(ColorRGBA.Red);
                slotsText.setColor(ColorRGBA.Red);

//                chatLog.addLog("Game solved!!!!", ChatLogController.SYSTEM);
                addCommand("$ system win " + playerID);

                gameSolved = true;
            }
        }
    }

    /**
     * Anwser is sort of truned up side down sidesVector ex:
     * 123456 - iterator
     * 251364 - sidesVector
     * 314625 - anwserVector
     */
    private void calcAnwserVector() {
        int[] sides = sidesVector;

        // iterate over the answer
        for (int i = 0; i < 6; i++) {
            // iterate over the sieds searching for value
            // then flip the valu with the interator
            for (int j = 0; j < 6; j++) {
                if (sides[j] - 1 == i) {
                    answerVector[i] = j + 1;
                }
            }
        }
    }

    public String getAnwserAsString() {
        String s = "answerVector=";
        for (int i : answerVector) {
            s = s + i;
        }
        return s;
    }

    public void addCommand(String command) {
        commandQueue.add(command);
    }

    private void commandSolve() {
        String command = "";
        while (!commandQueue.isEmpty()) {
            command = commandQueue.poll();

            /* The mesage was a command */
            if (command.startsWith("$ ")) {
                String[] args = command.split(" ");

//                if (debug) {
//                    for (String arg : args) {
//                        chatLog.addLog(arg, ChatLogController.DEBUG);
//                    }
//                }

                if (args[1].equals("help")) {
                    chatLog.addLog("[$ move x y] Move to xy", ChatLogController.HELP);
                    chatLog.addLog("[$ get Name] Interact with object", ChatLogController.HELP);
                    chatLog.addLog("[$ rot direction] Rotate inventory", ChatLogController.HELP);
                    chatLog.addLog("[$ opmove x y] Move opponent to xy", ChatLogController.HELP);
                    chatLog.addLog("[$ opget Name] Interact opponent with object", ChatLogController.HELP);
                    chatLog.addLog("[$ oprot direction] Rotate inventory", ChatLogController.HELP);
                    chatLog.addLog("[$ exit] Exit the game", ChatLogController.HELP);
                } else if (args[1].equals("whoami")) {
                    chatLog.addLog("playerID=" + playerID + " opponentID=" + opponentID, ChatLogController.SYSTEM);
                } else if (args[1].equals("opchat")) {
                    String chatMessage = "";
                    for (int i = 2; i < args.length; i++) {
                        chatMessage += args[i];
                        chatMessage += " ";
                    }

                    chatLog.addLog(chatMessage, ChatLogController.OPPONENT);
                } else if (args[1].equals("system")) {
                    String systemMessage = args[2];

                    if (systemMessage.equals("win")) {
                        int winnerID = Integer.parseInt(args[3]);

                        chatLog.addLog("Game solved!!!", ChatLogController.SYSTEM);

                        if (winnerID == playerID) {
                            chatLog.addLog("You won :)", ChatLogController.SYSTEM);
                        } else {
                            chatLog.addLog("You lost :(", ChatLogController.SYSTEM);
                        }
                    } else {
                        chatLog.addLog(systemMessage, ChatLogController.SYSTEM);
                    }
                } else if (args[1].equals("move")) {
                    float x = Float.parseFloat(args[2]);
                    float y = Float.parseFloat(args[3]);

                    Vector3f target = new Vector3f(x, 0, y);
                    player.move(target);
                    mc.move(playerID, x, y);
                } else if (args[1].equals("get")) {
                    String arg = args[2];
                    Node target = null;

                    if (arg.substring(0, 4).equals("Cube")) {
                        target = (Node) sceneNode.getChild(arg);

                        int cubeID = Integer.parseInt(arg.substring(4));
                        mc.cubeTake(playerID, cubeID);
                    } else if (arg.substring(0, 4).equals("Slot")) {
                        chatLog.addLog("You want to get a Slot.", ChatLogController.DEBUG);
                        target = (Node) player.getSlotsPhysNode().getChild(arg);

                        int cubeID = Integer.parseInt(arg.substring(4));
                        mc.cubeInsert(playerID, cubeID);
                    }

                    if (target != null) {
                        player.moveToGet(target);
                    }
                } else if (args[1].equals("rot")) {
                    if (args[2].equals("left")) {
                        player.inventoryLeft(tpf);
                        mc.cubeRotate(playerID, "left");
                    }
                    if (args[2].equals("right")) {
                        player.inventoryRight(tpf);
                        mc.cubeRotate(playerID, "right");
                    }
                    if (args[2].equals("up")) {
                        player.inventoryUp(tpf);
                        mc.cubeRotate(playerID, "up");
                    }
                    if (args[2].equals("down")) {
                        player.inventoryDown(tpf);
                        mc.cubeRotate(playerID, "down");
                    }

                } else if (args[1].equals("opmove")) {
                    float x = Float.parseFloat(args[2]);
                    float y = Float.parseFloat(args[3]);

                    Vector3f target = new Vector3f(x, 0, y);
                    opponent.move(target);

                } else if (args[1].equals("opget")) {
                    String arg = args[2];
                    Node target;

                    if (arg.substring(0, 4).equals("Slot")) {
                        chatLog.addLog("Opponent wants to get a Slot.", ChatLogController.DEBUG);
                        target = (Node) opponent.getSlotsPhysNode().getChild(arg);
                    } else {
                        target = (Node) sceneNode.getChild(arg);
                    }

                    if (target != null) {
                        opponent.moveToGet(target);
                    }

                } else if (args[1].equals("oprot")) {
                    if (args[2].equals("left")) {
                        opponent.inventoryLeft(tpf);
                    }
                    if (args[2].equals("right")) {
                        opponent.inventoryRight(tpf);
                    }
                    if (args[2].equals("up")) {
                        opponent.inventoryUp(tpf);
                    }
                    if (args[2].equals("down")) {
                        opponent.inventoryDown(tpf);
                    }

                } else if (args[1].equals("exit")) {
                    nifty.exit();
                    stopPuzzledMonkey();
                }


                /* User send a chat */
            } else if (!command.startsWith("$ ")) {
                chatLog.addLog(command, ChatLogController.YOU);
                mc.chat(playerID, command);
            }


        }

    }

    private void stopPuzzledMonkey() {
        mc.stop();
        stop();
    }
}
